package cn.ibizlab.core.analysis.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.beans.factory.annotation.Value;
import cn.ibizlab.util.errors.BadRequestAlertException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.core.analysis.domain.DAChart;
import cn.ibizlab.core.analysis.filter.DAChartSearchContext;
import cn.ibizlab.core.analysis.service.IDAChartService;

import cn.ibizlab.util.helper.CachedBeanCopier;
import cn.ibizlab.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.core.analysis.mapper.DAChartMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[图表] 服务对象接口实现
 */
@Slf4j
@Service("DAChartServiceImpl")
public class DAChartServiceImpl extends ServiceImpl<DAChartMapper, DAChart> implements IDAChartService {


    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(DAChart et) {
        if(!this.retBool(this.baseMapper.insert(et))) {
            return false;
        }
        CachedBeanCopier.copy(get(et.getChartId()), et);
        return true;
    }

    @Override
    @Transactional
    public void createBatch(List<DAChart> list) {
        this.saveBatch(list, batchSize);
    }

    @Override
    @Transactional
    public boolean update(DAChart et) {
        if(!update(et, (Wrapper) et.getUpdateWrapper(true).eq("chartid", et.getChartId()))) {
            return false;
        }
        CachedBeanCopier.copy(get(et.getChartId()), et);
        return true;
    }

    @Override
    @Transactional
    public void updateBatch(List<DAChart> list) {
        updateBatchById(list, batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result = removeById(key);
        return result ;
    }

    @Override
    @Transactional
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public DAChart get(String key) {
        DAChart et = getById(key);
        if (et == null) {
            throw new BadRequestAlertException("数据不存在", this.getClass().getSimpleName(), key);
        }
        else {
        }
        return et;
    }

    @Override
    public DAChart getDraft(DAChart et) {
        return et;
    }

    @Override
    public boolean checkKey(DAChart et) {
        return (!ObjectUtils.isEmpty(et.getChartId())) && (!Objects.isNull(this.getById(et.getChartId())));
    }
    @Override
    @Transactional
    public boolean save(DAChart et) {
        if(!saveOrUpdate(et)) {
            return false;
        }
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(DAChart et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? getProxyService().update(et) : getProxyService().create(et);
        }
    }

    @Override
    @Transactional
    public boolean saveBatch(Collection<DAChart> list) {
        List<DAChart> create = new ArrayList<>();
        List<DAChart> update = new ArrayList<>();
        for (DAChart et : list) {
            if (ObjectUtils.isEmpty(et.getChartId()) || ObjectUtils.isEmpty(getById(et.getChartId()))) {
                create.add(et);
            } else {
                update.add(et);
            }
        }
        if (create.size() > 0) {
            getProxyService().createBatch(create);
        }
        if (update.size() > 0) {
            getProxyService().updateBatch(update);
        }
        return true;
    }

    @Override
    @Transactional
    public void saveBatch(List<DAChart> list) {
        List<DAChart> create = new ArrayList<>();
        List<DAChart> update = new ArrayList<>();
        for (DAChart et : list) {
            if (ObjectUtils.isEmpty(et.getChartId()) || ObjectUtils.isEmpty(getById(et.getChartId()))) {
                create.add(et);
            } else {
                update.add(et);
            }
        }
        if (create.size() > 0) {
            getProxyService().createBatch(create);
        }
        if (update.size() > 0) {
            getProxyService().updateBatch(update);
        }
    }



    /**
     * 查询集合 数据集
     */
    @Override
    public Page<DAChart> searchDefault(DAChartSearchContext context) {
            com.baomidou.mybatisplus.extension.plugins.pagination.Page<DAChart> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
            return new PageImpl<DAChart>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }







    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }



    public IDAChartService getProxyService() {
        return cn.ibizlab.util.security.SpringContextHolder.getBean(this.getClass());
    }
}



