package cn.ibizlab.core.util.config;

import lombok.extern.slf4j.Slf4j;
import org.apache.rocketmq.client.exception.MQClientException;
import org.apache.rocketmq.client.producer.DefaultMQProducer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.autoconfigure.condition.ConditionalOnExpression;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

@Slf4j
@Configuration
public class MQProducerConfigure {
    public static final Logger LOGGER = LoggerFactory.getLogger(MQProducerConfigure.class);

    @Value("${rocketmq.producer.groupName:dstproducer}")
    private String groupName;
    @Value("${rocketmq.producer.namesrvAddr:}")
    private String namesrvAddr;
    // 消息最大值
    @Value("${rocketmq.producer.maxMessageSize:409600}")
    private Integer maxMessageSize;
    // 消息发送超时时间
    @Value("${rocketmq.producer.sendMsgTimeOut:3000}")
    private Integer sendMsgTimeOut;
    // 失败重试次数
    @Value("${rocketmq.producer.retryTimesWhenSendFailed:2}")
    private Integer retryTimesWhenSendFailed;

    /**
     * mq 生成者配置
     * @return
     * @throws MQClientException
     */
    @Bean
    @ConditionalOnExpression("${rocketmq.producer.enabled:false}")
    public DefaultMQProducer defaultProducer() throws MQClientException {
        log.info("defaultProducer 正在创建---------------------------------------");
        DefaultMQProducer producer = new DefaultMQProducer(groupName);
        producer.setNamesrvAddr(namesrvAddr);
        producer.setVipChannelEnabled(false);
        producer.setMaxMessageSize(maxMessageSize);
        producer.setSendMsgTimeout(sendMsgTimeOut);
        producer.setRetryTimesWhenSendAsyncFailed(retryTimesWhenSendFailed);
        producer.start();
        log.info("defaultProducer server 开启成功 groupName={}, namesrvAddr={}",groupName,namesrvAddr);
        return producer;
    }
}
